@@ -28,8 +28,9 @@
 #include "algorithm.h"
 #include "ansiescapecodehandler.h"
 #include "fileinprojectfinder.h"
+#include "link.h"
 #include "qtcassert.h"
-#include "synchronousprocess.h"
+#include "qtcprocess.h"
 #include "theme/theme.h"
 
 #include <QDir>
@@ -72,15 +73,14 @@ bool OutputLineParser::isLinkTarget(const QString &target)
     return target.startsWith(*linkPrefix());
 }
 
-void OutputLineParser::parseLinkTarget(const QString &target, FilePath &filePath, int &line,
-                                       int &column)
+Link OutputLineParser::parseLinkTarget(const QString &target)
 {
     const QStringList parts = target.mid(linkPrefix()->length()).split(*linkSep());
     if (parts.isEmpty())
-        return;
-    filePath = FilePath::fromString(parts.first());
-    line = parts.length() > 1 ? parts.at(1).toInt() : 0;
-    column = parts.length() > 2 ? parts.at(2).toInt() : 0;
+        return {};
+    return Link(FilePath::fromString(parts.first()),
+                parts.length() > 1 ? parts.at(1).toInt() : 0,
+                parts.length() > 2 ? parts.at(2).toInt() : 0);
 }
 
 // The redirection mechanism is needed for broken build tools (e.g. xcodebuild) that get invoked
@@ -134,10 +134,12 @@ bool OutputLineParser::demoteErrorsToWarnings() const
     return d->demoteErrorsToWarnings;
 }
 
-FilePath OutputLineParser::absoluteFilePath(const FilePath &filePath)
+FilePath OutputLineParser::absoluteFilePath(const FilePath &filePath) const
 {
-    if (filePath.isEmpty() || filePath.toFileInfo().isAbsolute())
+    if (filePath.isEmpty())
         return filePath;
+    if (filePath.toFileInfo().isAbsolute())
+        return filePath.cleanPath();
     FilePaths candidates;
     for (const FilePath &dir : searchDirectories()) {
         FilePath candidate = dir.pathAppended(filePath.toString());
@@ -217,6 +219,7 @@ public:
     bool boldFontEnabled = true;
     bool prependCarriageReturn = false;
     bool prependLineFeed = false;
+    bool forwardStdOutToStdError = false;
 };
 
 OutputFormatter::OutputFormatter() : d(new Private) { }
@@ -264,7 +267,7 @@ void OutputFormatter::addLineParser(OutputLineParser *parser)
 void OutputFormatter::setupLineParser(OutputLineParser *parser)
 {
     parser->setFileFinder(&d->fileFinder);
-    connect(parser, &OutputLineParser::newSearchDir, this, &OutputFormatter::addSearchDir);
+    connect(parser, &OutputLineParser::newSearchDirFound, this, &OutputFormatter::addSearchDir);
     connect(parser, &OutputLineParser::searchDirExpired, this, &OutputFormatter::dropSearchDir);
 }
 
@@ -299,7 +302,8 @@ void OutputFormatter::doAppendMessage(const QString &text, OutputFormat format)
     // then our formatting should reflect that redirection as well, i.e. print in red
     // even if the nominal format is stdout.
     if (!involvedParsers.isEmpty()) {
-        const OutputFormat formatForParser = outputTypeForParser(involvedParsers.last(), format);
+        const OutputFormat formatForParser = res.formatOverride
+                ? *res.formatOverride : outputTypeForParser(involvedParsers.last(), format);
         if (formatForParser != format && cleanLine == text && formattedText.length() == 1) {
             charFmt = charFormat(formatForParser);
             formattedText.first().format = charFmt;
@@ -321,7 +325,7 @@ void OutputFormatter::doAppendMessage(const QString &text, OutputFormat format)
         if (d->postPrintAction)
             d->postPrintAction(p);
         else
-            p->runPostPrintActions();
+            p->runPostPrintActions(plainTextEdit());
     }
 }
 
@@ -329,7 +333,9 @@ OutputLineParser::Result OutputFormatter::handleMessage(const QString &text, Out
                                                         QList<OutputLineParser *> &involvedParsers)
 {
     // We only invoke the line parsers for stdout and stderr
-    if (format != StdOutFormat && format != StdErrFormat)
+    // Bad: on Windows we may get stdout and stdErr only as DebugFormat as e.g. GUI applications
+    // print them Windows-internal and we retrieve this separately
+    if (format != StdOutFormat && format != StdErrFormat && format != DebugFormat)
         return OutputLineParser::Status::NotHandled;
     const OutputLineParser * const oldNextParser = d->nextParser;
     if (d->nextParser) {
@@ -494,6 +500,7 @@ void OutputFormatter::initFormats()
     d->formats[StdOutFormat].setForeground(theme->color(Theme::OutputPanes_StdOutTextColor));
     d->formats[StdErrFormat].setForeground(theme->color(Theme::OutputPanes_StdErrTextColor));
     d->formats[DebugFormat].setForeground(theme->color(Theme::OutputPanes_DebugTextColor));
+    d->formats[GeneralMessageFormat].setForeground(theme->color(Theme::OutputPanes_DebugTextColor));
     setBoldFontEnabled(d->boldFontEnabled);
 }
 
@@ -525,12 +532,10 @@ bool OutputFormatter::handleFileLink(const QString &href)
 {
     if (!OutputLineParser::isLinkTarget(href))
         return false;
-    FilePath filePath;
-    int line;
-    int column;
-    OutputLineParser::parseLinkTarget(href, filePath, line, column);
-    QTC_ASSERT(!filePath.isEmpty(), return false);
-    emit openInEditorRequested(filePath, line, column);
+
+    Link link = OutputLineParser::parseLinkTarget(href);
+    QTC_ASSERT(!link.targetFilePath.isEmpty(), return false);
+    emit openInEditorRequested(link);
     return true;
 }
 
@@ -573,6 +578,11 @@ void OutputFormatter::setBoldFontEnabled(bool enabled)
     d->formats[ErrorMessageFormat].setFontWeight(fontWeight);
 }
 
+void OutputFormatter::setForwardStdOutToStdError(bool enabled)
+{
+    d->forwardStdOutToStdError = enabled;
+}
+
 void OutputFormatter::flush()
 {
     if (!d->incompleteLine.first.isEmpty())
@@ -582,7 +592,7 @@ void OutputFormatter::flush()
     for (OutputLineParser * const p : qAsConst(d->lineParsers))
         p->flush();
     if (d->nextParser)
-        d->nextParser->runPostPrintActions();
+        d->nextParser->runPostPrintActions(plainTextEdit());
 }
 
 bool OutputFormatter::hasFatalErrors() const
@@ -607,7 +617,7 @@ void OutputFormatter::dropSearchDir(const FilePath &dir)
 OutputFormat OutputFormatter::outputTypeForParser(const OutputLineParser *parser,
                                                   OutputFormat type) const
 {
-    if (type == StdOutFormat && parser->needsRedirection())
+    if (type == StdOutFormat && (parser->needsRedirection() || d->forwardStdOutToStdError))
         return StdErrFormat;
     return type;
 }
@@ -628,7 +638,7 @@ void OutputFormatter::appendMessage(const QString &text, OutputFormat format)
         d->prependCarriageReturn = false;
         out.prepend('\r');
     }
-    out = SynchronousProcess::normalizeNewlines(out);
+    out = QtcProcess::normalizeNewlines(out);
     if (out.endsWith('\r')) {
         d->prependCarriageReturn = true;
         out.chop(1);
